#include <main.hpp>

/************************************************
 *              Global Variables                *
 ************************************************/
RemoteVariable aqi = RemoteVariable();
ESPMegaPRO espmega = ESPMegaPRO();
const char *mode_names[] = AC_MODE_NAMES;
const char *fan_speed_names[] = AC_FAN_SPEED_NAMES;
AirConditioner ac_config = {
    .max_temperature = 32,
    .min_temperature = 16,
    .modes = 4,
    .mode_names = mode_names,
    .fan_speeds = 4,
    .fan_speed_names = fan_speed_names,
    .getInfraredCode = &getInfraredCode};

ClimateCard ac = ClimateCard(AC_IR_TX_PIN, ac_config, AC_SENSOR_TYPE, AC_SENSOR_PIN, AC_IR_CHANNEL);

/************************************************
 *            End of Global Variables           *
 ************************************************/

void setup()
{
  // Initialize both displayports
  INTERNAL_DISPLAY_UART.begin(INTERNAL_DISPLAY_BAUDRATE);
  DISPLAY_UART.begin(DISPLAY_COMMUNICATION_BAUDRATE, SERIAL_8N1, DISPLAY_RX, DISPLAY_TX);
  // Reset both display
  send_stop_bytes(INTERNAL_DISPLAY_UART);
  INTERNAL_DISPLAY_UART.print("rest");
  send_stop_bytes(INTERNAL_DISPLAY_UART);
  send_stop_bytes(DISPLAY_UART);
  DISPLAY_UART.print("rest");
  send_stop_bytes(DISPLAY_UART);
  // Wait for the display to reset
  vTaskDelay(1000 / portTICK_PERIOD_MS);
  // Initialize the ESPMegaPRO
  espmega.begin();
  // Initialize IoT Modules
  espmega.enableIotModule();
  // Setup Ethernet
  ETH.begin();
  espmega.iot->bindEthernetInterface(&ETH);
  // Connect to Network
  espmega.iot->loadNetworkConfig();
  espmega.iot->connectNetwork();
  // Connect to the MQTT Broker
  espmega.iot->loadMqttConfig();
  espmega.iot->connectToMqtt();
  // Enable Web Server
  espmega.enableWebServer(80);
  // Initialize Internal Display
  espmega.enableInternalDisplay(&INTERNAL_DISPLAY_UART);
  // Initialize Air Conditioner
  espmega.installCard(2, &ac);
  ac.bindFRAM(&espmega.fram, AC_FRAM_ADDR);
  espmega.display->bindClimateCard(&ac); // Show our climate card on the display
  // Register all cards with iot
  espmega.iot->registerCard(0);
  espmega.iot->registerCard(1);
  espmega.iot->registerCard(2);
  // Initialize Remote Variables
  aqi.begin(10, AQI_STATE_TOPIC, espmega.iot, true, AQI_REQUEST_TOPIC);
  // Input callbacks
  // This pre-load the input buffers
  // We need to do this to prevent switches that are left on the "on" position from triggering the callback
  espmega.inputs.loop();
  espmega.inputs.registerCallback(handle_input_change);
  // Start the display routine
  // TODO - Write CUDDisplay class
}

void loop()
{
  espmega.loop();
}

void send_stop_bytes(HardwareSerial &uart)
{
  uart.write(0xFF);
  uart.write(0xFF);
  uart.write(0xFF);
}

void handle_input_change(uint8_t pin, bool state) {
  // When input pin 0 - 6 changes, toggle the corresponding output pin
  if (pin >= 0 && pin <= 6) {
    espmega.outputs.setState(pin, !espmega.outputs.getState(pin));
  }
}